#include <stdio.h>
#include "Field.h"

CField::CField(CDisplay* ig_pDisplay, int ilevel)
{
	// 
	g_pDisplay=ig_pDisplay;
	memset(block,0,sizeof(int)*6*8);

	level=ilevel;
	player_anim=false;
	player_x=0;
	player_v=0;
	player_speed=1;
	field_x=25;
	field_y=25;
	fire_y=-1;
	score=0;
	open_count=0;

	changed_bg=true;

	add_time=0;
	time=timeGetTime();
	need_show_fps=true;

	//   
	g_pDisplay->CreateSurface(&surface_text,88,18);
	surface_text->Clear(0xFFFFFF);
	//  
	char filename[64];
	sprintf(filename,"spr/bg%d.bmp",level);
	spr_background=new CSprite(g_pDisplay,0,0,640,480,filename);
	sprintf(filename,"spr/fieldd%d.bmp",level);
	spr_fieldd=new CSprite(g_pDisplay,field_x,field_y,206,73,filename);
	sprintf(filename,"spr/fieldu%d.bmp",level);
	spr_fieldu=new CSprite(g_pDisplay,field_x,field_y,206,357,filename);
	spr_block=new CSprite(g_pDisplay,0,0,80,80,"spr/block.bmp");
	for (int i=0;i<6;i++)
	{
		sprintf(filename,"spr/fire%d.bmp",i+1);
		aspr_fire[i]=new CSprite(g_pDisplay,0,0,64,64,0xFF00FF,filename);
		sprintf(filename,"spr/fire%d_1.bmp",i+1);
		aspr_fire_anim[i]=new CSprite(g_pDisplay,0,0,64,64,0xFF00FF,filename);
		for (int j=2;j<=9;j++) //    . 
		{
			sprintf(filename,"spr/fire%d_%d.bmp",i+1,j);
			aspr_fire_anim[i]->AddFrame(filename);
		}
		aspr_fire_anim[i]->SetAnimationDelay(10);
		aspr_fire_anim[i]->SetLoopCount(1);
	}
	LineClear();
	FClear();
}


CField::~CField()
{
	//   
	delete surface_text;
	//  
	delete spr_background;
	delete spr_block;
	delete spr_fieldu;
	delete spr_fieldd;
	for (int i=0;i<6;i++)
		delete aspr_fire[i];
}

void CField::BlockOpen(void)
{
	int i,j,random_count=0;
	changed_bg=true;
	do
	{
		i=rand()%6;
		j=rand()%8;
		random_count++;
		if (random_count>1000)
		{
			for (i=0;i<6;i++)
				for (j=0;j<8;j++)
					if (block[i][j]==0)
					{
						block[i][j]=1;
						return;
					} /* of if */
			//  !!!!!!
			doWin();
			return;
		}
	} /* of do */
	while (block[i][j]!=0);
	block[i][j]=1;
}

// 
void CField::doWin(void)
{
	while (field_y<480)
	{
		int step=field_y/5;
		if (step<1) step=1;
		field_y+=step;
		changed_field=true;
		changed_bg=true;
		Show();
		g_pDisplay->Present();
	}
}

// 
void CField::doDie(void)
{
	while (field_x<640)
	{
		int step=field_x/5;
		if (step<1) step=1;
		field_x+=step;
		changed_field=true;
		changed_bg=true;
		Show();
		g_pDisplay->Present();
	}
}


bool CField::IsFinished(void)
{
	if (field_y<480 && field_x<640) return false; else return true;
}

void CField::Show(void)
{
	//  
	if (changed_bg)
	{
		spr_background->Show();
		for (int i=0;i<6;i++)
			for (int j=0;j<8;j++)
				if (block[i][j]==0) spr_block->Show(j*80,i*80);
	changed_bg=false;
	}
	spr_fieldd->Show(field_x,field_y+357);

	//     %32
	if (player_speed==1) {
		int need_add_x;
		if (last_direction== 1) need_add_x=32-(player_x%32);
		if (last_direction==-1) need_add_x=(player_x%32);
		if (need_add_x!=32 && need_add_x!=0)
		{
			int chg=need_add_x/2;
			if (chg<1) chg=1;
			player_x+=last_direction*chg;
			if (player_x<0) player_x=0;
			if (player_x>160) player_x=160;
		}
	}
	
	//   
	if (player_anim)
	{
		int pv=player_v-1;
		if (pv<0) pv=5;
		aspr_fire_anim[pv]->Show(player_x+field_x-9,field_y+341);
		if (aspr_fire_anim[pv]->IsAlreadyAnimated())
		{
			aspr_fire_anim[pv]->Rewind(0);
			player_anim=false;
		}
	}
	else
	{
		aspr_fire[player_v]->Show(player_x+field_x-9,field_y+341);
	}

	FDraw(field_x-9,field_y-11);
	int curent_time=FLineAdd(2000);
	LineDraw(field_x-9,field_y+377);

	//  
	if (fire_y>=0)
	{
		changed_field=true;
		if (f[fire_y][fire_x]!=0) //   
		{
			if (f[fire_y][fire_x]==fire_v+1) //    
			{
				LineAddTo(f[fire_y][fire_x]);
				f[fire_y][fire_x]=0;
			}
			fire_y=-1;
		}
		if (fire_y>=0)
			aspr_fire[fire_v]->Show(fire_x*32+field_x-9,field_y+fire_y*32-11);
		fire_y--;
		if (fire_y<0) fire_y=-1;
	}

	//  FPS
	if (need_show_fps)
	{
		char fps[32];
		if ((curent_time-time)==0)
			sprintf(fps,"FPS: TO MUCH");
		else
			sprintf(fps,"FPS: %d   ",1000/(curent_time-time));
		time=curent_time;
		font_big->Draw(0,0,fps);
	}
	
}



// ####################################################################
//     #############################################
//  
void CField::FClear(void)
{
	memset(f,0,sizeof(int)*12*6);
	changed_field=true;
}

//   (  )
int CField::FLineAdd(long itime)
{
	int curent_time=timeGetTime();
	add_time+=curent_time-time;
	if (add_time>itime)
	{		
		changed_field=true;
		add_time=0;
		for (int j=10;j>=0;j--)
			memcpy(f[j+1],f[j],sizeof(int)*6);

		for (int i=0;i<6;i++)
			f[0][i]=1+rand()%6;
	}
	return curent_time;
}

//  count   .
void CField::FRemove(int count)
{
	changed_field=true;
	for (int j=10;j>=0;j--)
		for (int i=5;i>=0;i--)
			if (count>0 && f[j][i]!=0)
			{
				f[j][i]=0;
				count--;
			}
}

//   
bool CField::FIsDead(void)
{
	bool die=false;
	for (int i=0;i<6;i++) if (f[11][i]!=0) die=true;
	return die;
}

//  
void CField::FDraw(int ix, int iy)
{
	if (!changed_field) return;
	changed_field=false;
	spr_fieldu->Show(ix+9,iy+11);
	for (int i=0;i<12;i++)
		for (int j=0;j<6;j++) 
			if (f[i][j]!=0)
				aspr_fire[f[i][j]-1]->Show(j*32+ix,i*32+iy);
}


// ####################################################################
//   () ##################################
//    ""
void CField::key_Right(void)
{
	player_x+=player_speed;
	if (player_speed<4) player_speed+=1;
	if (player_x>160) player_x=160;
	last_direction=1;
}

//    ""
void CField::key_Left(void)
{
	player_x-=player_speed;
	if (player_speed<4) player_speed+=1;
	if (player_x<0) player_x=0;
	last_direction=-1;
}

//    ""
void CField::key_Up(void)
{
	if (player_anim) return;
	player_anim=true;
	player_v++;
	if (player_v>5) player_v=0;	
}

//    ""
void CField::key_Fire(void)
{
	if (player_anim) return;
	if (player_x%32==0 && fire_y==-1)
	{
		fire_y=10;
		fire_x=player_x/32;
		fire_v=player_v;
	}
}

//    ""
void CField::key_Down(void)
{
	int remove=LineCheck();
	if (remove!=0)
	{
		FRemove(remove);
		LineClear();
		score=score+remove;
		while (score/SCORE_PER_BLOCK>open_count)
		{
			open_count++;
			BlockOpen();
		}
	}
}

//      
void CField::key_Release(void)
{
	player_speed=1;
}



// ####################################################################
//     #############################################
//    
void CField::LineAddTo(int i)
{
	for (int j=1;j<6;j++) line[j-1]=line[j];
	line[5]=i;
}

//    
//    .
int CField::LineCheck(void)
{
	if (line[0]==0 || line[1]==0 || line[2]==0 || line[3]==0 || line[4]==0 || line[5]==0) return(0);
	if (line[0]==1 && line[1]==2 && line[2]==3 && line[3]==4 && line[4]==5 && line[5]==6) return(24);
	if (line[0]==6 && line[1]==5 && line[2]==4 && line[3]==3 && line[4]==2 && line[5]==1) return(24);
	if (line[0]==line[1] && line[0]==line[1] && line[0]==line[2] && line[0]==line[3] && line[0]==line[4] && line[0]==line[5]) return(20);
	if (line[0]==line[1] && line[2]==line[3] && line[4]==line[5]) return(16);
	if (line[0]==line[1] && line[1]==line[2])
		if (line[3]==line[4] && line[4]==line[5])
			return(12);
	return(0);
}

//  
void CField::LineClear(void)
{
	memset(line,0,sizeof(int)*6);
}

//  
void CField::LineDraw(int x, int y)
{
	for (int i=0;i<6;i++)
		if (line[i]!=0)
			aspr_fire[line[i]-1]->Show(i*32+x,y);
};

