#include "Sprite.h"

// 
CSprite::~CSprite()
{
	for (int i=0;i<frame_total;i++)
		SAFE_DELETE( surface[i] );
}

//  (CDisplay -  , x, y, , ,  BMP   ID BMP  )
CSprite::CSprite(CDisplay* iwhere_display, int ix, int iy, int iwidth, int iheight, TCHAR* strBMP)
{
	where_display=iwhere_display;
	frame_total=frame_curent=0;
	colorkey=0;
	memset(animation_delay,0,sizeof(int)*MAX_SPRITE_FRAMES);
	last_time=0;
	loop_total=0;
	loop_curent=0;
	x=ix;
	y=iy;
	width=iwidth;
	height=iheight;
	visible=true;
    if (FAILED(where_display->CreateSurfaceFromBitmap( &surface[frame_total], strBMP, width, height)))
	{
        MessageBox( 0, "Failed sprite creating.", strBMP, MB_ICONERROR | MB_OK );
		failed=true;
		return;
	}
	frame_total++;
	failed=false;
}

//  (CDisplay -  , x, y,  BMP   ID BMP  )
CSprite::CSprite(CDisplay* iwhere_display, int ix, int iy, TCHAR* strBMP)
{
	where_display=iwhere_display;
	frame_total=frame_curent=0;
	colorkey=0;
	memset(animation_delay,0,sizeof(int)*MAX_SPRITE_FRAMES);
	last_time=0;
	loop_total=0;
	loop_curent=0;
	x=ix;
	y=iy;
	visible=true;
    if (FAILED(where_display->CreateSurfaceFromBitmap( &surface[frame_total], strBMP, 0, 0)))
	{
        MessageBox( 0, "Failed sprite creating.", strBMP, MB_ICONERROR | MB_OK );
		failed=true;
		return;
	}
	width=0;
	height=0;
	frame_total++;
	failed=false;
}

//  (CDisplay -  , x, y, , ,  ,  BMP   ID BMP  )
CSprite::CSprite(CDisplay* iwhere_display, int ix, int iy, int iwidth, int iheight, int icolorkey, TCHAR* strBMP)
{
	where_display=iwhere_display;
	frame_total=frame_curent=0;
	colorkey=icolorkey;
	memset(animation_delay,0,sizeof(int)*MAX_SPRITE_FRAMES);
	last_time=0;
	x=ix;
	y=iy;
	loop_total=0;
	loop_curent=0;
	width=iwidth;
	height=iheight;
	visible=true;
    if (FAILED(where_display->CreateSurfaceFromBitmap( &surface[frame_total], strBMP, width, height)))
	{
        MessageBox( 0, "Failed sprite creating.", strBMP, MB_ICONERROR | MB_OK );
		failed=true;
		return;
	}
	surface[frame_total]->SetColorKey(colorkey);
	frame_total++;
	failed=false;
}

//    
void CSprite::AddFrame(TCHAR* strBMP)
{
	if (failed) return;
	if (frame_total<MAX_SPRITE_FRAMES)
	{
		if (FAILED(where_display->CreateSurfaceFromBitmap( &surface[frame_total], strBMP, width, height)))
			return;
		else
		{
			animation_delay[frame_total]=0;
			if (frame_total>0)
				if (surface[frame_total-1]->IsColorKeyed())
					surface[frame_total]->SetColorKey(colorkey);
			frame_total++;
		}
	}
}

//   
void CSprite::SetColorKey(int color)
{
	if (failed) return;
	for (int i=0;i<frame_total;i++)
	{
		surface[i]->SetColorKey(color);
	}
}

//     where_display
void CSprite::Show(void)
{
	if (visible && !failed)
	{
	    int dwCurrTick = timeGetTime();
		RECT r;
		r.left=0;
		r.top=0;
		r.right=width;
		r.bottom=height;
		if (x+width>WINDOW_WIDTH) r.right=WINDOW_WIDTH-x;
		if (y+height>WINDOW_HEIGHT) r.bottom=WINDOW_HEIGHT-y;

		where_display->Blt(x, y, surface[frame_curent], &r);
		if (dwCurrTick-last_time>animation_delay[frame_curent])
		{
			frame_curent++;
			if (frame_curent>=frame_total)
			{
				if (loop_total==0)
				{
					frame_curent=0;
					loop_curent=0;
				}
				else
				{
					if (loop_curent<loop_total)
					{
						frame_curent=0;
						loop_curent++;
					}
					else frame_curent=frame_total-1;
				}
			}
		last_time=dwCurrTick;
		}
	}
}

//     where_display
void CSprite::Show(int ix, int iy)
{
	x=ix;
	y=iy;
	Show();
}



//   X 
void CSprite::SetX(int ix)
{
	x=ix;
}

//   Y 
void CSprite::SetY(int iy)
{
	y=iy;
}

//   
void CSprite::doHide(void)
{
	visible=false;
}

//  
void CSprite::doShow(void)
{
	visible=true;
}

//     
void CSprite::SetAnimationDelay(int delay)
{
	for (int i=0;i<frame_total;i++)
		animation_delay[i]=delay;
}

//     
void CSprite::SetAnimationFPS(int fps)
{
	for (int i=0;i<frame_total;i++)
		animation_delay[i]=1000/fps;
}

//      
void CSprite::SetAnimationDelay(int delay, int frame_num)
{
	if (frame_num<frame_total)
		animation_delay[frame_num]=delay;
}

//      
void CSprite::SetAnimationFPS(int fps, int frame_num)
{
	if (frame_num<frame_total)
		animation_delay[frame_num]=1000/fps;
}

//       
// 0 -  
void CSprite::SetLoopCount(int icount)
{
	loop_total=icount;
}

//        , 
//  true
bool CSprite::IsAlreadyAnimated(void)
{
	if (loop_total==0) return false;
	if (loop_curent==loop_total-1 && frame_curent==frame_total-1) return true;
	else return false;
}

//    
void CSprite::Rewind(int iframe)
{
	loop_curent=0;
	frame_curent=iframe;
	if (iframe>=frame_total) frame_curent=frame_total-1;
}
