#define STRICT
#include <windows.h>
#include <ddraw.h>
#include <dinput.h>
#include <mmsystem.h>
#include "resource.h"
#include "dx\ddutil.h"
#include "sprite.h"
#include "field.h"
#include "Mfont.h"

LPDIRECTINPUT8       g_pDI       = NULL;
LPDIRECTINPUTDEVICE8 g_pKeyboard = NULL;

CDisplay*            g_pDisplay        = NULL;
RECT                 g_rcViewport;          
RECT                 g_rcScreen;            
BOOL                 g_bActive     = FALSE; 
CField* f;
CMfont* font_big;

//-----------------------------------------------------------------------------
// Function-prototypes
//-----------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HRESULT WinInit( HINSTANCE hInst, int nCmdShow, HWND* phWnd, HACCEL* phAccel );
//   DirectX  DirectDraw
HRESULT InitDirectDraw(HWND hWnd);
VOID    FreeDirectDraw();
HRESULT DisplayFrame(HWND hWnd);
HRESULT RestoreSurfaces();
//   DirectInput
HRESULT OnCreateDevice( HWND hDlg );
HRESULT ReadImmediateData( HWND hDlg );
VOID    FreeDirectInput();




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything and calls
//       UpdateFrame() when idle from the message pump.
//-----------------------------------------------------------------------------
int APIENTRY WinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR pCmdLine, int nCmdShow )
{
    MSG		 msg;
    HWND     hWnd;
    HACCEL   hAccel;

    srand( GetTickCount() );

    if( FAILED( WinInit( hInst, nCmdShow, &hWnd, &hAccel ) ) )
        return FALSE;

    if( FAILED( InitDirectDraw( hWnd ) ) )
    {
        MessageBox( hWnd, "DirectDraw init failed.", "DirectDraw", MB_ICONERROR | MB_OK );
        return FALSE;
    }

    while( TRUE )
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                return (int)msg.wParam;
            }

            if( 0 == TranslateAccelerator( hWnd, hAccel, &msg ) )
            {
                TranslateMessage( &msg ); 
                DispatchMessage( &msg );
            }
        }

	DisplayFrame(hWnd);
    }
}





//-----------------------------------------------------------------------------
// Name: WinInit()
// Desc: Init the window
//-----------------------------------------------------------------------------
HRESULT WinInit( HINSTANCE hInst, int nCmdShow, HWND* phWnd, HACCEL* phAccel )
{
    WNDCLASSEX wc;
    HWND       hWnd;
    HACCEL     hAccel;

    // Register the Window Class
    wc.cbSize        = sizeof(wc);
    wc.lpszClassName = TEXT("WindowedMode");
    wc.lpfnWndProc   = MainWndProc;
    wc.style         = CS_VREDRAW | CS_HREDRAW;
    wc.hInstance     = hInst;
    wc.hIcon         = LoadIcon( hInst, MAKEINTRESOURCE(IDI_MAIN) );
    wc.hIconSm       = LoadIcon( hInst, MAKEINTRESOURCE(IDI_MAIN) );
    wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wc.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU);
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;

    if( RegisterClassEx( &wc ) == 0 )
        return E_FAIL;

    // Load keyboard accelerators
    hAccel = LoadAccelerators( hInst, MAKEINTRESOURCE(IDR_MAIN_ACCEL) );

    // Calculate the proper size for the window given a client of 640x480
    DWORD dwFrameWidth    = GetSystemMetrics( SM_CXSIZEFRAME );
    DWORD dwFrameHeight   = GetSystemMetrics( SM_CYSIZEFRAME );
    DWORD dwMenuHeight    = GetSystemMetrics( SM_CYMENU );
    DWORD dwCaptionHeight = GetSystemMetrics( SM_CYCAPTION );
    DWORD dwWindowWidth   = WINDOW_WIDTH  + dwFrameWidth * 2;
    DWORD dwWindowHeight  = WINDOW_HEIGHT + dwFrameHeight * 2 + 
                            dwMenuHeight + dwCaptionHeight;

    // Create and show the main window
    DWORD dwStyle = WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX;
    hWnd = CreateWindowEx( 0, TEXT("WindowedMode"), TEXT("DirectDraw WindowedMode Sample"),
                           dwStyle, CW_USEDEFAULT, CW_USEDEFAULT,
  	                       dwWindowWidth, dwWindowHeight, NULL, NULL, hInst, NULL );
    if( hWnd == NULL )
    	return E_FAIL;

    ShowWindow( hWnd, nCmdShow );
    UpdateWindow( hWnd );

    *phWnd   = hWnd;
    *phAccel = hAccel;

    return S_OK;
}





//-----------------------------------------------------------------------------
// Name: InitDirectDraw()
// Desc: Create the DirectDraw object, and init the surfaces
//-----------------------------------------------------------------------------
HRESULT InitDirectDraw( HWND hWnd )
{
    LPDIRECTDRAWPALETTE pDDPal = NULL; 
    HRESULT	hr;

    g_pDisplay = new CDisplay();
    if( FAILED( hr = g_pDisplay->CreateWindowedDisplay(hWnd, WINDOW_WIDTH, WINDOW_HEIGHT ) ) )
//    if( FAILED( hr = g_pDisplay->CreateFullScreenDisplay(hWnd, WINDOW_WIDTH, WINDOW_HEIGHT,24 ) ) )
    {
        MessageBox( hWnd, "Failed initializing.","DirectDraw", MB_ICONERROR | MB_OK );
        return hr;
    }
	if( NULL == g_pKeyboard )
	{
		if( FAILED(OnCreateDevice(hWnd)))
		{
			MessageBox( hWnd, "CreateDevice() failed","Keyboard", MB_ICONERROR | MB_OK );
			FreeDirectInput();
		}
	}
	else FreeDirectInput();

	// 
	f=new CField(g_pDisplay,1);
	font_big=new CMfont();
	font_big->LoadFont("spr/font.txt","spr/font.bmp",g_pDisplay);
	font_big->SetColorKey(0x00013C);

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FreeDirectDraw()
// Desc: Release all the DirectDraw objects
//-----------------------------------------------------------------------------
VOID FreeDirectDraw()
{
    SAFE_DELETE(f);
	SAFE_DELETE(font_big);
    SAFE_DELETE(g_pDisplay);
}





//-----------------------------------------------------------------------------
// Name: MainWndProc()
// Desc: The main window procedure
//-----------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch (msg)
    {
        case WM_COMMAND: //////////////////////////////////////////////
            switch( LOWORD(wParam) )
            {
                case IDM_EXIT:
            	    PostMessage( hWnd, WM_CLOSE, 0, 0 );
                    return 0L;
            }
            break;
        case WM_QUERYNEWPALETTE: //////////////////////////////////////
            if( g_pDisplay )
            {
                LPDIRECTDRAWPALETTE pDDPal = NULL; 
                g_pDisplay->GetFrontBuffer()->GetPalette( &pDDPal );
                g_pDisplay->GetFrontBuffer()->SetPalette( pDDPal );
                SAFE_RELEASE( pDDPal );
            }
            break;

        case WM_GETMINMAXINFO: ////////////////////////////////////////
            {
                MINMAXINFO* pMinMax = (MINMAXINFO*) lParam;

                DWORD dwFrameWidth    = GetSystemMetrics( SM_CXSIZEFRAME );
                DWORD dwFrameHeight   = GetSystemMetrics( SM_CYSIZEFRAME );
                DWORD dwMenuHeight    = GetSystemMetrics( SM_CYMENU );
                DWORD dwCaptionHeight = GetSystemMetrics( SM_CYCAPTION );

                pMinMax->ptMinTrackSize.x = WINDOW_WIDTH  + dwFrameWidth * 2;
                pMinMax->ptMinTrackSize.y = WINDOW_HEIGHT + dwFrameHeight * 2 + 
                                            dwMenuHeight + dwCaptionHeight;

                pMinMax->ptMaxTrackSize.x = pMinMax->ptMinTrackSize.x;
                pMinMax->ptMaxTrackSize.y = pMinMax->ptMinTrackSize.y;
            }
            return 0L;

        case WM_MOVE: /////////////////////////////////////////////////
	        if( g_pDisplay )
		        g_pDisplay->UpdateBounds();
            return 0L;

        case WM_SIZE: /////////////////////////////////////////////////
            if( SIZE_MAXHIDE==wParam || SIZE_MINIMIZED==wParam )
                g_bActive = FALSE;
            else
                g_bActive = TRUE;

	        if( g_pDisplay )
		        g_pDisplay->UpdateBounds();
            break;

        case WM_PAINT:
			DisplayFrame(hWnd);
			break;

		case WM_DESTROY: //////////////////////////////////////////////
			KillTimer(hWnd,0);
            FreeDirectDraw();
            PostQuitMessage(0);
            return 0L;
    }

    return DefWindowProc(hWnd, msg, wParam, lParam);
}







//-----------------------------------------------------------------------------
// Name: DisplayFrame()
// Desc: Blts a the sprites to the back buffer, then it blts or flips the 
//       back buffer onto the primary buffer.
//-----------------------------------------------------------------------------
HRESULT DisplayFrame(HWND hWnd)
{
    HRESULT hr;
	if (!f) return S_OK;
	if (f->FIsDead()) f->doDie();
	if (f->IsFinished())
	{
		SendMessage(hWnd,WM_DESTROY,0,0);
	}
	else
	{
		if (FAILED(ReadImmediateData(hWnd)))
		{
			MessageBox(NULL, "Error reading input state.","Keyboard", MB_ICONERROR | MB_OK );
		}

		f->Show();
		if(FAILED( hr = g_pDisplay->Present())) return hr;
	}
    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RestoreSurfaces()
// Desc: Restore all the surfaces, and redraw the sprite surfaces.
//-----------------------------------------------------------------------------
HRESULT RestoreSurfaces()
{
    LPDIRECTDRAWPALETTE pDDPal = NULL; 
    HRESULT hr;

    if( FAILED( hr = g_pDisplay->GetDirectDraw()->RestoreAllSurfaces() ) )
        return hr;

    return S_OK;
}





///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Name: OnCreateDevice()
// Desc: Setups a the keyboard device using the flags from the dialog.
//-----------------------------------------------------------------------------
HRESULT OnCreateDevice( HWND hDlg )
{
    HRESULT hr;
    DWORD   dwCoopFlags;
#ifdef _WIN64
    HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr( hDlg, GWLP_HINSTANCE );
#else
    HINSTANCE hInst = (HINSTANCE) GetWindowLong( hDlg, GWL_HINSTANCE );
#endif

    KillTimer( hDlg, 0 );    
    FreeDirectInput();

    dwCoopFlags = DISCL_NONEXCLUSIVE;
    dwCoopFlags |= DISCL_FOREGROUND;

    if( FAILED( hr = DirectInput8Create( GetModuleHandle(NULL), DIRECTINPUT_VERSION, 
                                         IID_IDirectInput8, (VOID**)&g_pDI, NULL ) ) )
        return hr;
    
    if( FAILED( hr = g_pDI->CreateDevice( GUID_SysKeyboard, &g_pKeyboard, NULL ) ) )
        return hr;
    
    if( FAILED( hr = g_pKeyboard->SetDataFormat( &c_dfDIKeyboard ) ) )
        return hr;
    
    hr = g_pKeyboard->SetCooperativeLevel( hDlg, dwCoopFlags );
    if( hr == DIERR_UNSUPPORTED)
    {
        FreeDirectInput();
        MessageBox( hDlg, "SetCooperativeLevel() returned DIERR_UNSUPPORTED.\nFor security reasons, background exclusive keyboard\n access is not allowed.", "Keyboard", MB_OK );
        return S_OK;
    }

    if (FAILED(hr)) return hr;

    g_pKeyboard->Acquire();

    return S_OK;
}



//-----------------------------------------------------------------------------
// Name: ReadImmediateData()
// Desc: Read the input device's state when in immediate mode and display it.
//-----------------------------------------------------------------------------
HRESULT ReadImmediateData( HWND hDlg )
{
    HRESULT hr;
    BYTE    diks[256];
    int     i;
	int		kpressed;

    if( NULL == g_pKeyboard ) 
        return S_OK;
    
    ZeroMemory( &diks, sizeof(diks) );
    hr = g_pKeyboard->GetDeviceState( sizeof(diks), &diks );
    if( FAILED(hr) ) 
    {
        hr = g_pKeyboard->Acquire();
        while( hr == DIERR_INPUTLOST ) 
            hr = g_pKeyboard->Acquire();
        return S_OK; 
    }
    
	kpressed=0;
    for( i = 0; i < 256; i++ ) 
    {
		if (diks[i]!=0)
		{
			kpressed=1;
			if (i==0xCD) f->key_Right();
			if (i==0xCB) f->key_Left();
			if (i==0xC8) f->key_Up();
			if (i==0xD0) f->key_Down();
			if (i==0x39) f->key_Fire();
		}
    }
	if (kpressed==0) f->key_Release();

    return S_OK;
}



//-----------------------------------------------------------------------------
// Name: FreeDirectInput()
// Desc: Initialize the DirectInput variables.
//-----------------------------------------------------------------------------
VOID FreeDirectInput()
{
    if (g_pKeyboard) g_pKeyboard->Unacquire();
    
    SAFE_RELEASE( g_pKeyboard );
    SAFE_RELEASE( g_pDI );
}









